/* -*- C++ -*-
 * test/test_tools.h - Part of LEMON, a generic C++ optimization library
 *
 * Copyright (C) 2005 Egervary Jeno Kombinatorikus Optimalizalasi Kutatocsoport
 * (Egervary Research Group on Combinatorial Optimization, EGRES).
 *
 * Permission to use, modify and distribute this software is granted
 * provided that this copyright notice appears in all copies. For
 * precise terms see the accompanying LICENSE file.
 *
 * This software is provided "AS IS" with no warranty of any kind,
 * express or implied, and with no claim as to its suitability for any
 * purpose.
 *
 */

#ifndef LEMON_TEST_TEST_TOOLS_H
#define LEMON_TEST_TEST_TOOLS_H

#include <iostream>
#include <vector>

#include <cstdlib>
#include <ctime>

#include <lemon/invalid.h>
#include <lemon/concept_check.h>

using namespace lemon;

//! \ingroup misc
//! \file
//! \brief Some utilities to write test programs.


///If \c rc is fail, writes an error message end exit.

///If \c rc is fail, writes an error message end exit.
///The error message contains the file name and the line number of the
///source code in a standard from, which makes it possible to go there
///using good source browsers like e.g. \c emacs.
///
///For example
///\code check(0==1,"This is obviously false.");\endcode will
///print this (and then exits).
///\verbatim graph_test.cc:123: error: This is obviously false. \endverbatim
///
///\todo It should be in \c error.h
#define check(rc, msg) \
  if(!(rc)) { \
    std::cerr << __FILE__ ":" << __LINE__ << ": error: " << msg << std::endl; \
    exit(1); \
  } else { } \

///Structure returned by \ref addPetersen().

///Structure returned by \ref addPetersen().
///
template<class Graph> struct PetStruct
{
  ///Vector containing the outer nodes.
  std::vector<typename Graph::Node> outer;
  ///Vector containing the inner nodes.
  std::vector<typename Graph::Node> inner;
  ///Vector containing the edges of the inner circle.
  std::vector<typename Graph::Edge> incir;
  ///Vector containing the edges of the outer circle.
  std::vector<typename Graph::Edge> outcir;
  ///Vector containing the chord edges.
  std::vector<typename Graph::Edge> chords;
};



///Adds a Petersen graph to \c G.

///Adds a Petersen graph to \c G.
///\return The nodes and edges of the generated graph.

template<typename Graph>
PetStruct<Graph> addPetersen(Graph &G,int num = 5)
{
  PetStruct<Graph> n;

  for(int i=0;i<num;i++) {
    n.outer.push_back(G.addNode());
    n.inner.push_back(G.addNode());
  }

 for(int i=0;i<num;i++) {
   n.chords.push_back(G.addEdge(n.outer[i],n.inner[i]));
   n.outcir.push_back(G.addEdge(n.outer[i],n.outer[(i+1) % num]));
   n.incir.push_back(G.addEdge(n.inner[i],n.inner[(i+2) % num]));
  }
 return n;
}

/// \brief Adds to the graph the reverse pair of all edges.
///
/// Adds to the graph the reverse pair of all edges.
///
template<class Graph> void bidirGraph(Graph &G)
{
  typedef typename Graph::Edge Edge;
  typedef typename Graph::EdgeIt EdgeIt;
  
  std::vector<Edge> ee;
  
  for(EdgeIt e(G);e!=INVALID;++e) ee.push_back(e);

  for(typename std::vector<Edge>::iterator p=ee.begin();p!=ee.end();p++)
    G.addEdge(G.target(*p),G.source(*p));
}


/// \brief Checks the bidirectioned Petersen graph.
///
///  Checks the bidirectioned Petersen graph.
///
template<class Graph> void checkBidirPetersen(Graph &G, int num = 5)
{
  typedef typename Graph::Node Node;

  typedef typename Graph::EdgeIt EdgeIt;
  typedef typename Graph::NodeIt NodeIt;

  checkGraphNodeList(G, 2 * num);
  checkGraphEdgeList(G, 6 * num);

  for(NodeIt n(G);n!=INVALID;++n) {
    checkGraphInEdgeList(G, n, 3);
    checkGraphOutEdgeList(G, n, 3);
  }  
}

///Structure returned by \ref addUndirPetersen().

///Structure returned by \ref addUndirPetersen().
///
template<class Graph> struct UndirPetStruct
{
  ///Vector containing the outer nodes.
  std::vector<typename Graph::Node> outer;
  ///Vector containing the inner nodes.
  std::vector<typename Graph::Node> inner;
  ///Vector containing the edges of the inner circle.
  std::vector<typename Graph::UndirEdge> incir;
  ///Vector containing the edges of the outer circle.
  std::vector<typename Graph::UndirEdge> outcir;
  ///Vector containing the chord edges.
  std::vector<typename Graph::UndirEdge> chords;
};

///Adds a Petersen graph to the undirected \c G.

///Adds a Petersen graph to the undirected \c G.
///\return The nodes and edges of the generated graph.

template<typename Graph>
UndirPetStruct<Graph> addUndirPetersen(Graph &G,int num=5)
{
  UndirPetStruct<Graph> n;

  for(int i=0;i<num;i++) {
    n.outer.push_back(G.addNode());
    n.inner.push_back(G.addNode());
  }

 for(int i=0;i<num;i++) {
   n.chords.push_back(G.addEdge(n.outer[i],n.inner[i]));
   n.outcir.push_back(G.addEdge(n.outer[i],n.outer[(i+1)%5]));
   n.incir.push_back(G.addEdge(n.inner[i],n.inner[(i+2)%5]));
  }
 return n;
}

int _urandom_init() {
  int seed = time(0);
  srand(seed);
  return seed;
}

int urandom(int n) {
  static int seed = _urandom_init();
  ignore_unused_variable_warning(seed);
  return (int)(rand() / (1.0 + RAND_MAX) * n);
}

#endif
